##########################################################################
# R code for constructing the function betai.nlme used to obtain         #
# standard errors of individual predicted PMI                            #
# Manuscript: Fitting non-linear mixed models to messy longitudinal data #
# Authors: Francisco M.M. Rocha, Carmen D.S. Andre and Julio M Singer    #
# Updated May 21, 2014                                                   #
##########################################################################

betai.nlme = function (fit) {
  require(MASS) 
  require(Matrix)
  require(car)

######################################################################################

extract.lmeDesign2 <- function(m){
    start.level = 1
    
    data <- if(any(!complete.cases(m$data))){
      warning("Removing incomplete cases from supplied data.") 
      m$data[complete.cases(m$data),]
    } else m$data
    grps <- nlme::getGroups(m)
    n <- length(grps)
    X <- list()
    grp.dims <- m$dims$ncol
    Zt <- model.matrix(m$modelStruct$reStruct, data)
    cov <- as.matrix(m$modelStruct$reStruct)
    i.col <- 1
    n.levels <- length(m$groups)
    Z <- matrix(0, n, 0)
    if (start.level <= n.levels) {
      for (i in 1:(n.levels - start.level + 1)) {
        if(length(levels(m$groups[[n.levels-i+1]]))!=1)
        {
          X[[1]] <- model.matrix(~m$groups[[n.levels - i +
                                              1]] - 1, 
                                 contrasts.arg = c("contr.treatment",
                                                   "contr.treatment"))
        }
        else X[[1]]<-matrix(1, n, 1)
        X[[2]] <- as.matrix(Zt[, i.col:(i.col + grp.dims[i] -
                                          1)])
        i.col <- i.col + grp.dims[i]
        Z <- cbind(mgcv::tensor.prod.model.matrix(X),Z)
      }
      Vr <- matrix(0, ncol(Z), ncol(Z))
      start <- 1
      for (i in 1:(n.levels - start.level + 1)) {
        k <- n.levels - i + 1
        for (j in 1:m$dims$ngrps[i]) {
          stop <- start + ncol(cov[[k]]) - 1
          Vr[ncol(Z)+1-(stop:start),ncol(Z)+1-(stop:start)] <- cov[[k]]
          start <- stop + 1
        }
      }
    }
    X <- if(class(m$call$fixed) == "name" &&  !is.null(m$data$X)){
      m$data$X
    } else   {
      model.matrix(formula(eval(m$call$fixed)),data)
    }
    y<-as.vector(matrix(m$residuals, ncol=NCOL(m$residuals))[,NCOL(m$residuals)] + 
                   matrix(m$fitted, ncol=NCOL(m$fitted))[,NCOL(m$fitted)])
    return(list(
      Vr=Vr, 
      X=X,
      Z=Z,
      sigmasq=m$sigma^2,
      lambda=unique(diag(Vr)),
      y=y,
      k=n.levels
    )
    )
  }

#####################################################################################

data.fit<-extract.lmeDesign2(fit)
data<-fit$data
X<- data.fit$X
y<- data.fit$y
id<- as.numeric(names(getResponse(fit)))
subject<-as.numeric(unique(id))
n<-length(as.numeric(names(table(id)))) 
N<-length(y)   


######################################################################################
mataux<-model.matrix(fit$modelStruct$reStruct, data)
mataux<- as.data.frame(cbind(mataux,id))
for(i in (as.numeric(unique(id)))){ 
  ifelse( i==min(as.numeric(unique(id))), 
          Z<- as.matrix((subset(split(mataux,id==min(id),drop=T)$`TRUE`,select=-id))),
          Z<-as.matrix(bdiag(Z,as.matrix(subset(split(mataux,id==i,drop=T)$`TRUE`,select=-id)))))
}

# Computation of matrices underlying the covariance structure

# Estimate of the G matrix 
G<- getVarCov(fit, type="random.effects")

# Estimate of the Gamma matrix
q<-dim(G)[1]                          
Gamma<-as.matrix(kronecker(diag(length(as.numeric(unique(id)))), G))

iGamma<-solve(Gamma)

# Estimate of the R matrix 
R <- getVarCov(fit, type="conditional", individual=1)[[1]]
for (i in 2:length(as.numeric(unique(id)))){
  R <- as.matrix(bdiag(R, getVarCov(fit, type="conditional", individual=i)[[1]] ) )
  
}

iR<-solve(R)

# Construction of the V matrix (covariance matrix of Y)
V<- (Z%*%Gamma%*%t(Z)) + R
iV<-ginv(V)                       


# Construction of the Q matrix 
varbeta<- ginv((t(X)%*%iV%*%X))
Q<-(iV-iV%*%X%*%(varbeta)%*%t(X)%*%iV )  

# EBLUE and EBLUP
eblue<-as.matrix(fixef(fit))
eblup<-Gamma%*%t(Z)%*%iV%*%(y-X%*%eblue)

# Standard errors of the individual intercepts and slopes alpha+a_i and beta+b_i
cov.eblup.eblue<- (-1)*varbeta%*%t(X)%*%iV%*%Z%*%Gamma # Cov(beta, b)
cov.eblup<- Gamma%*%t(Z)%*%Q%*%Z%*%Gamma
cov.pred.error<-  Gamma - cov.eblup

aux1<- cbind(varbeta, cov.eblup.eblue)
aux2<- cbind(t(cov.eblup.eblue), cov.pred.error)
mat.cov.eblue.eblup<-rbind(aux1,aux2)

vector.eblue.eblup<-rbind((eblue),eblup)

# Contast matrix K
K<-cbind((matrix(1,n,1)%x%diag(1,q)),diag(1,(q*n)))

# Student t statistic
K.beta_ind<- (K)%*%vector.eblue.eblup
matcov.K.beta.ind<- (K)%*%mat.cov.eblue.eblup%*%t(K)
se.K.beta.ind<- sqrt(diag((K)%*%mat.cov.eblue.eblup%*%t(K)))
tobs<-K.beta_ind/se.K.beta.ind
table(tobs>2)

## Table with results
parameter<- rep(names(fixef(fit)),n)
Subject<-rep(subject, each=2)
table.result<-data.frame(Subject, parameter, 
                      K.beta_ind, se.K.beta.ind,tobs)
colnames(table.result)<-c("Subject", "Parameter", "Estimate", "Std.Error", "t-value")

Result<- list(Table=table.result, Matrix=matcov.K.beta.ind)

}

###############################################################################################################
