##############################################################################################################
# R code for fitting exponential models to the hypostasis data                                               #
# Manuscript: Fitting non-linear mixed models to messy longitudinal data: a case studey involving estimation #
#             of post mortem intervlas                                                                       #
# Authors: Francisco M.M. Rocha, Carmen D.S. Andre, Julio M Singer and Talita Zerbini                        #
# Updated June 14, 2017                                                                                      #
##############################################################################################################

library(MASS)
library(lattice)
library(latticeExtra)
library(nlme)
library(car)
library(gdata)
library(Matrix)

#####################################################################################################
# To obtain diagnostic plots, run the fuction residdiag_nlme_v37.R available from                   #
# www.ime.usp.br/~jmsinger/lmmdiagnostics.zip)                                                      #
# With Rstudio, this may be carried out via the command: source, e.g.,                              #
# source("C:/jmsinger/Desktop/residdiag_nlme_v37.R")                                                #
# To obtain the standard errors of the PMI, run the function: betai.nlme (included in the zip file) #
#####################################################################################################

# Read data from Excel spreadsheet and define transformed variables
hypostasis<-read.xls("/home/jmsinger/Desktop/hypostasis.xls", sheet='data', method="tab")

hypostasis$logdifARPL <- log(hypostasis$difARPL)
hypostasis$invPMIe <- 1/hypostasis$PMIe
hypostasis$Subject2 <- as.factor(hypostasis$Subject)
hypostasis$obs<-1:64

# Descriptive plots
xyplot(difARPL~PMIe,  data=hypostasis, group=Subject2, type =c("l", "p"), strip=FALSE)
xyplot(difARPL~PMIe | Subject2, data=hypostasis, type="b", strip=TRUE)
xyplot(logdifARPL~invPMIe | Subject2, data=hypostasis, type="b", strip=TRUE)
xyplot(logdifARPL~invPMIe, groups=Subject2 , data=hypostasis, type="b",table=T)


#############################################################################################################

# Exponential model with random intercept
fit1<-lme(logdifARPL ~ invPMIe, data=hypostasis, random = ~ 1  | Subject, 
           method="REML", na.action=na.omit) 
summary(fit1)
# To obtain 95% confidence intervals for all model parameters, use the command: intervals(fit1)

# Diagnostic plots (residuals)
res1<-residdiag.nlme(fit1, limit=2, plotid=1:16)

# Individual predicted plots
pred <- with(hypostasis, expand.grid(invPMIe=seq(min(invPMIe), 1, length.out=40),
             Subject=as.character(unique(Subject))))

pred$logdifARPLSS.fit1 <- predict(fit1, newdata=pred, level=1)
pred$logdifARPLPA.fit1 <- predict(fit1, newdata=pred, level=0)

pred$difARPLPA.fit1 <- exp(pred$logdifARPLPA.fit1)
pred$difARPLSS.fit1 <- exp(pred$logdifARPLSS.fit1)
pred$PMIe <- 1/pred$invPMIe

xyplot(difARPL~PMIe|Subject2, data=hypostasis, par.settings=standard.theme(quartz,color=F), 
       key=list(columns=2,text=list(c("PA", "SS")), lines=list(type="l",col=c("black","black"),lty=1:2)),
       xlab="PMI",ylab="DifARPL",ylim=c(0,100)) +
       as.layer(xyplot(pred$difARPLSS.fit1~PMIe|Subject, data=pred, type="l", col=1, lty=2)) +
       as.layer(xyplot(pred$difARPLPA.fit1~PMIe|Subject, data=pred, type="l", col=1, lty=1))

#############################################################################################################

# Exponential model with random slope
fit2<-lme(logdifARPL ~ invPMIe, data=hypostasis, random = ~ invPMIe-1  | Subject, na.action=na.omit) 
summary(fit2)

# Diagnostic plots (residuals)
res2<-residdiag.nlme(fit2, limit=2, plotid=1:16)

# Individual predicted plots
pred <- with(hypostasis, expand.grid(invPMIe=seq(min(invPMIe), 1, length.out=40),
             Subject=as.character(unique(Subject))))

pred$logdifARPLSS.fit2 <- predict(fit2, newdata=pred, level=1)
pred$logdifARPLPA.fit2 <- predict(fit2, newdata=pred, level=0)

pred$difARPLPA.fit2 <- exp(pred$logdifARPLPA.fit2)
pred$difARPLSS.fit2 <- exp(pred$logdifARPLSS.fit2)
pred$PMIe <- 1/pred$invPMIe

xyplot(difARPL~PMIe|Subject2, data=hypostasis,par.settings=standard.theme(quartz,color=F), 
       key=list(columns=2,text=list(c("PA", "SS")), lines=list(type="l",col=c("black","black"),lty=1:2)),
       xlab="PMI",ylab="DifARPL",ylim=c(0,100)) +
       as.layer(xyplot(pred$difARPLSS.fit2~PMIe|Subject, data=pred, type="l", col=1,,lty=2)) +
       as.layer(xyplot(pred$difARPLPA.fit2~PMIe|Subject, data=pred, type="l", col=1,lty=1))

##############################################################################################################

# Exponential model with random intercept and slope
fit3<-lme(logdifARPL ~ invPMIe, data=hypostasis, random = ~ invPMIe  | Subject, na.action=na.omit) 
summary(fit3)

# Diagnostic plots
res3<-residdiag.nlme(fit3, limit=2, plotid=1:16)


## Individual predicted plots
pred <- with(hypostasis, expand.grid(invPMIe=seq(min(invPMIe), 1, length.out=40),
             Subject=as.character(unique(Subject))))

pred$logdifARPLSS.fit3 <- predict(fit3, newdata=pred, level=1)
pred$logdifARPLPA.fit3 <- predict(fit3, newdata=pred, level=0)

pred$difARPLPA.fit3 <- exp(pred$logdifARPLPA.fit3)
pred$difARPLSS.fit3 <- exp(pred$logdifARPLSS.fit3)
pred$PMIe <- 1/pred$invPMIe

xyplot(difARPL~PMIe|Subject2, data=hypostasis, par.settings=standard.theme(quartz,color=F), 
       key=list(columns=2,text=list(c("PA", "SS")), lines=list(type="l",col=c("black","black"),lty=1:2)),
       xlab="PMI",ylab="DifARPL",ylim=c(0,100)) +
       as.layer(xyplot(pred$difARPLSS.fit3~PMIe|Subject, data=pred, type="l", col=1,,lty=2)) +
       as.layer(xyplot(pred$difARPLPA.fit3~PMIe|Subject, data=pred, type="l", col=1, lty=1))

################################################################################################################

# Exponential model with random intercept and slope and different variances for cases 10, 12, 13 and 14 

hypostasis$group10and12and13and14 <- I(as.numeric( I(hypostasis$Subject2 == 10 )) +
                                     2*as.numeric( I(hypostasis$Subject2 == 12 )) +
                                     3*as.numeric( I(hypostasis$Subject2 == 13 )) + 
                                     4*as.numeric( I(hypostasis$Subject2 == 14 )) )
fit3a <- lme(logdifARPL ~ invPMIe, data=hypostasis, random = ~ invPMIe  | Subject, na.action=na.omit,
             weights=varIdent(form = ~ 1 | group10and12and13and14))
summary(fit3a)

# Diagnostic plots
res3a<-residdiag.nlme(fit3a, limit=2, plotid=1:16)

#################################################################################################################

# Exponential model with random intercept and slope and different variances for cases 10, 12, 13, 14 and 15, 
# with exclusion of observation 13.1

hypostasis1 <- hypostasis[ which((hypostasis$Subject2 != '13' | hypostasis$PMIe != 1) ),]

hypostasis1$group10and12and13and14and15_13.1 <- I(as.numeric( I(hypostasis1$Subject2 == 10 )) +
                                                2*as.numeric( I(hypostasis1$Subject2 == 12 )) +
                                                3*as.numeric( I(hypostasis1$Subject2 == 13 )) + 
                                                4*as.numeric( I(hypostasis1$Subject2 == 14 )) +
                                                5*as.numeric( I(hypostasis1$Subject2 == 15 )) )

fit3b <- lme(logdifARPL ~ invPMIe, data=hypostasis1, random = ~ invPMIe  | Subject, na.action=na.omit,
             weights=varIdent(form = ~ 1 | group10and12and13and14and15_13.1))
summary(fit3b)

# Diagnostic plots
res3b<-residdiag.nlme(fit3b, limit=2, plotid=1:16)

#################################################################################################################

# Exponential model with random intercept and slope and different variances for cases 10, 12, 14 and 15, 
# with exclusion of unit 13

hypostasis2 <- hypostasis[ which((hypostasis$Subject2 != '13' ) ),]

hypostasis2$group10and12and13and14and15_13 <- I(as.numeric( I(hypostasis2$Subject2 == 10 )) +
                                              2*as.numeric( I(hypostasis2$Subject2 == 12 )) +
                                              3*as.numeric( I(hypostasis2$Subject2 == 14 )) +
                                              4*as.numeric( I(hypostasis2$Subject2 == 15 )) )

fit3c <- lme(logdifARPL ~ invPMIe, data=hypostasis2, random = ~ invPMIe  | Subject, na.action=na.omit,
             weights=varIdent(form = ~ 1 | group10and12and13and14and15_13))
summary(fit3c)

# Diagnostic plots
res3c<-residdiag.nlme(fit3c, limit=2, plotid=1:16)

#################################################################################################################

# Exponential model with random intercept and slope and different variances for cases 10, 12, 14 and 15, 
# with exclusion of unit 13 and observation 15.1

hypostasis3 <- hypostasis[ which((hypostasis$Subject2 != '13' ) & (hypostasis$Subject2 != '15' | hypostasis$PMIe != 3)),]

hypostasis3$group10and12and14and15_13_15.1 <- I(as.numeric( I(hypostasis3$Subject2 == 10 )) +
                                               2*as.numeric( I(hypostasis3$Subject2 == 12 )) +
                                               3*as.numeric( I(hypostasis3$Subject2 == 14 )) +
                                               4*as.numeric( I(hypostasis3$Subject2 == 15 )) )

fit3d <- lme(logdifARPL ~ invPMIe, data=hypostasis3, random = ~ invPMIe  | Subject, na.action=na.omit,
             weights=varIdent(form = ~ 1 | group10and12and14and15_13_15.1))
summary(fit3d)

# Diagnostic plots
res3d<-residdiag.nlme(fit3d, limit=2, plotid=1:16)

##################################################################################################################

# Exponential model with random intercept and slope and different variances for cases 10, 12, 14 and 15, 
# with exclusion of unit 13 and observations 2.1 and 15.1

hypostasis4 <- hypostasis[ which((hypostasis$Subject2 != '13' ) & (hypostasis$Subject2 != '15' | hypostasis$PMIe != 3)
                                 & (hypostasis$Subject2 != '2' | hypostasis$PMIe != 3)),]

hypostasis4$group10and12and14and15_2.1_13_15.1 <- I(as.numeric( I(hypostasis4$Subject2 == 10 )) +
                                                  2*as.numeric( I(hypostasis4$Subject2 == 12 )) +
                                                  3*as.numeric( I(hypostasis4$Subject2 == 14 )) +
                                                  4*as.numeric( I(hypostasis4$Subject2 == 15 )) )

fit3e <- lme(logdifARPL ~ invPMIe, data=hypostasis4, random = ~ invPMIe  | Subject, na.action=na.omit,
             weights=varIdent(form = ~ 1 | group10and12and14and15_2.1_13_15.1))
summary(fit3e)

# Diagnostic plots
res3e<-residdiag.nlme(fit3e, limit=2, plotid=1:16)

###################################################################################################################
###################################################################################################################

# Predicted PMI (obtained via function: betai.nlme)

indivinform<-betai.nlme(fit3e)

var.betab.ind<-indivinform$Matrix  
eblue.eblup<-indivinform$Table

# Estimates of PMI

hypostasis.aux<-hypostasis[, c(1, 7)]

ind<-(unique(hypostasis.aux[,1]))
pred.x<-matrix(0, dim(hypostasis)[1],1)
k<-0
for(i in ind){ 
  aux1<-subset(hypostasis.aux, hypostasis.aux$Subject==i)
  aux2<-subset(eblue.eblup, eblue.eblup$Subject==i)
  n<-dim(aux1)[1]
  aux3<-matrix(0,n,1)
  for(j in 1:n){
    aux3[j]<- aux2[2,3]/(aux1[j,2]-aux2[1,3])
    k<-k+1
  }
  pred.x[((k-n)+1) :k,]<-aux3
}

A<-cbind(hypostasis.aux[,1:2],pred.x)
B<-cbind(hypostasis.aux[,1:1],pred.x); cbind(A,B)

# Derivatives dx/da and dx/db

k<-0
w<-0
mat.deriv<-matrix(0, dim(hypostasis)[1], dim(var.betab.ind)[1])
for(i in ind){ 
  aux11<-subset(hypostasis.aux, hypostasis.aux$Subject==i)
  aux22<-subset(eblue.eblup, eblue.eblup$Subject==i)
  n<-dim(aux11)[1]
  aux33<-matrix(0,n,2)
  w<-w+1
  for(j in 1:n){
    aux33[j,1]<- aux22[2,3]/((aux11[j,2]-aux22[1,3])^2)
    aux33[j,2]<- 1/(aux11[j,2]-aux22[1,3])  
    k<-k+1
  }
  mat.deriv[((k-n)+1):k,(2*w-1):(2*w)]<-aux33
}

## Standard errors of estimated PMI computed via the Delta method
var.time<- mat.deriv%*%var.betab.ind%*%t(mat.deriv)
std.error<-sqrt(diag(var.time))
mat.result<-cbind(hypostasis$Subject, pred.x, as.matrix(std.error), (pred.x-1.96*as.matrix(std.error)), 
                  (pred.x+1.96*as.matrix(std.error)))

table.result<-data.frame(mat.result)
colnames(table.result)<-c("Subject", "Estimate", "Std.Error", "Lower limit", "Upper limit")
table.result

################################################################################################################

## Individual predicted plots

pred <- with(hypostasis3, expand.grid(invPMIe=seq(min(invPMIe), 1, length.out=40),
                                     Subject=as.character(unique(Subject))))

pred$logdifARPLSS.fit3d <- predict(fit3d, newdata=pred, level=1)
pred$logdifARPLPA.fit3d <- predict(fit3d, newdata=pred, level=0)

pred$difARPLPA.fit3d <- exp(pred$logdifARPLPA.fit3d)
pred$difARPLSS.fit3d <- exp(pred$logdifARPLSS.fit3d)
pred$PMIe <- 1/pred$invPMIe

xyplot(PMIe~difARPL | Subject2, data=hypostasis3, par.settings=standard.theme(quartz, color=F), as.table = TRUE, 
       key=list(columns=2, text=list(c("PA", "SS")), lines=list(type="l", col=c("black","black"), lty=1:2)),
       xlab="difARPL", ylab="Predicted PMIe", ylim=c(0,25), xlim=c(0,75)) +
  as.layer(xyplot(PMIe~pred$difARPLSS.fit3d|Subject, data=pred, type="l", col=1, lty=2)) +
  as.layer(xyplot(PMIe~pred$difARPLPA.fit3d|Subject, data=pred, type="l", col=1, lty=1))

##################################################################################################################
