##########################################################################
# R code for fitting SEGMENTED REGRESSION models to the hypostasis data  #
# Manuscript: Fitting non-linear mixed models to messy longitudinal data #
# Authors: Francisco M.M. Rocha, Carmen D.S. Andre and Julio M Singer    #
# Updated May 21, 2014                                                   #
##########################################################################


library(lattice)
library(latticeExtra)
library(nlme)
library(gdata)
#==============================================================================#
#
# Set the working directory
# setwd("~/Documents/Artigos/BJPS/ComandosR")
#==============================================================================#
# Read data from Excel spreadsheet
# 
hypostasis <- read.xls("/home/jmsinger/Desktop/hypostasis.xls", sheet='data', method="tab")
#
# hypostasis <- read.xls("hypostasis.xls", sheet = 'data', method = "tab")
hypostasis$subject <- as.integer(hypostasis$Subject)


#==============================================================================#

# lme control
Ctrl <- list(msMaxEval = 100000,
             maxIter = 1e+08,maxit = 1e+08,
             pnlsMaxIter = 1e+08,
             msMaxIter = 100000,
             tolerance = 0.00001,
#             msTol = 0.00001,
             pnlsTol = 0.00001,
             niterEM = 1000,
             opt = "nlminb",optimMethod =  NULL ,  #'BFGS',      #   'L-BFGS-B',
#             factr = 1e7, #reltol = NULL,abstol = NULL,
#             pgtol = 0, 
             minScale = 1e-20,
             gradient = T,hessian = TRUE,
             msVerbose = T
)



#==============================================================================#
# Values minimum and maximum of the variable PMIe
a1 <- min(hypostasis$PMIe) # Menor valor do PMIe observado
a2 <- max(hypostasis$PMIe) # Maior valor do PMIe observado

#------------------------------------------------------------------------------#
# Step 0: Initialize

# 1: Setting an initial value for the changepoint \widetilde{\psi}_i = psii = psi
# for all i = 1,2,...,n
psi <- 9
#
# 2: Initializing the variables \widetilde{U} = U = (t-psi) I (t_{ij} <= psi_i) and 
hypostasis$U <- ifelse(hypostasis$PMIe <= psi,(hypostasis$PMIe ), psi)
#
# Fitting zero model

fit <- lme(difARPL ~  U - 1, random = ~  U - 1|subject, data = hypostasis)
summary(fit)
rand.coefpred <- (fit$coefficients)

# 
#------------------------------------------------------------------------------#
# Step 2:
# Setting psi = 8 = psii  and calculate deltai, aprox.lambda0im aprox G0i and O
# Lambda and psi
lambda <- log((psi - a1)/(a2 - psi))
psi <- (a1 + a2*exp(lambda))/(1 + exp(lambda)) 
# beta
beta <- as.numeric(rand.coefpred$fixed[1])
betai <- beta + ranef(fit)[,1]
Betai <- rep( betai,times = as.numeric(table(fit$groups)) ) 
# Matrix hij, g, offset
hij <- (1 - I(as.numeric(hypostasis$PMIe <= psi)))
g <- (exp(lambda)*(a2 - a1))/((1 + exp(lambda))^2)
hypostasis$G0 <- as.numeric(Betai*hij*g )
Offset <- -lambda*hypostasis$G0 
hypostasis$difARPLnew <- hypostasis$difARPL - Offset
# Matrix U 
hypostasis$U <- ifelse(hypostasis$PMIe <= psi, hypostasis$PMIe  , psi) 
hypostasis <- groupedData(difARPL ~  U  |Subject, data = hypostasis)
# Fit a model yij = betai(psi(lambda) + (xij - psi(lambda))I(xij <= psi(lambda)))
# + eij
fit <- lme(difARPLnew ~  U + G0 - 1 , random = ~  U - 1 |subject,
           control = Ctrl,method = 'ML' ,data = hypostasis)
summary(fit)
rand.coefpred <- (fit$coefficients)



#------------------------------------------------------------------------------#
# Step 3:

# Setting psi = 8 = psii  and calculate deltai, aprox.kappa0im aprox G0i and O
# Lambda and psi
lambda <- as.numeric(rand.coefpred$fixed[2])
psi <- (a1 + a2*exp( lambda ))/(1 + exp(lambda))
# beta
beta <- as.numeric(rand.coefpred$fixed[1])
betai <- beta + ranef(fit)[,1]
Betai <- rep( betai,times = as.numeric(table(fit$groups)) ) 
# Matrix hij, g, offset
hij <- (1 - I(as.numeric(hypostasis$PMIe <= psi)))
g <- (exp(lambda)*(a2 - a1))/((1 + exp(lambda))^2)
hypostasis$G0 <- as.numeric(Betai*hij*g )
Offset <- -lambda*hypostasis$G0 
hypostasis$difARPLnew <- hypostasis$difARPL - Offset
# Matrix U 
#  U 
hypostasis$U <- ifelse(hypostasis$PMIe <= psi, hypostasis$PMIe , psi)

epsilon <- 1
LogLik0 <- abs(fit$logLik)
iter <- 0

#------------------------------------------------------------------------------#
# Step 3:
while (abs(epsilon) > 1e-3) {
  
  fit <- lme(difARPLnew ~ U  + G0 - 1, random = ~ U - 1 |subject,
             control = Ctrl,method = 'REML',data = hypostasis)
  summary(fit)
  rand.coefpred <- (fit$coefficients)

  # New values
  # Lambda and psi 
  lambda <- as.numeric(rand.coefpred$fixed[2])
  psi <- (a1 + a2*exp( lambda ))/(1 + exp(lambda))
  # beta
  beta <- as.numeric(rand.coefpred$fixed[1])
  betai <- beta + ranef(fit)[,1]
  Betai <- rep( betai,times = as.numeric(table(fit$groups)) ) 
  # Matrix hij, g, offset
  hij <- (1 - I(as.numeric(hypostasis$PMIe <= psi)))
  g <- (exp(lambda)*(a2 - a1))/((1 + exp(lambda))^2)
  hypostasis$G0 <- as.numeric(Betai*hij*g )
  Offset <- -lambda*hypostasis$G0 
  hypostasis$difARPLnew <- hypostasis$difARPL - Offset
  # Matrix  U 
  hypostasis$U <- ifelse(hypostasis$PMIe <= psi, hypostasis$PMIe , psi)
  LogLik <- abs(fit$logLik)
  epsilon <-  abs(LogLik - LogLik0) / LogLik0
  LogLik0 <- LogLik
    cat( "\n")
    cat("iteration = ", iter <- iter + 1,'epsilon= ',epsilon,'psi= ',psi, "\n")
}
#==============================================================================#
#
# beta, lambda, psi, asymp and Var(asymp) 
psi\widehat{\kappa}_i\widehat{\kappa}_i
asymp <- beta*psi
asympi <- betai*psi
# Asymptote gradient  asymp(widehat(beta),psi(widehat(lambda)))
grad.asymp <- as.matrix(c(psi,beta*g),1,2)
# Asymp variance
var.asymp <- t(grad.asymp) %*% fit$varFix %*% grad.asymp
std.error <- sqrt(var.asymp)

#==============================================================================#
#==============================================================================#

# Individual predicted plots
# 
# 

m0 <- fit
cutpoint <- psi
a1new <- min(hypostasis$difARPLnew)
a2new <- max(hypostasis$difARPLnew)
pred <- with(hypostasis,
             expand.grid(xij = seq(1, a2new,by = 0.5),
                         subject = (unique(subject))))
pred$U <- ifelse(pred$xij <= psi,pred$xij,psi)

pred$G0 <- rep(0,dim(pred)[1]) 

# Commands to include zero slope line after cutpoint

pred$difARPL <- ifelse(pred$U <= cutpoint,
                       predict(m0,newdata = pred,level = 1), 0)
pred$subjectf <- as.factor(pred$subject)
xyplot(PMIe ~difARPL | factor(subject), data = hypostasis,strip = T,
       xlim = c(0,80),ylim = c(0,25),
       ,par.settings = standard.theme(quartz,color = F),as.table = TRUE,) +
  as.layer(xyplot(pred$xij~ difARPL|subjectf, data = pred, type = "l", lty=2,
                  col = 1,order = T))


