##############################################################################################################
# R code for fitting segmented regression models to the hypostasis data                                      #
# Manuscript: Fitting non-linear mixed models to messy longitudinal data: a case studey involving estimation #
#             of post mortem intervlas                                                                       #
# Authors: Francisco M.M. Rocha, Carmen D.S. Andre, Julio M Singer and Talita Zerbini                        #
# Updated June 14, 2017                                                                                      #
##############################################################################################################

library(lattice)
library(latticeExtra)
library(nlme)
library(gdata)

# Read data from Excel spreadsheet
hypostasis <- read.xls("hypostasis.xls", sheet = 'data', method = "tab")

hypostasis$subject <- as.integer(hypostasis$Subject)


##############################################################################################################
# lme control parameters

Ctrl <- list(msMaxEval = 100000,
             maxIter = 100000,maxit = 100000,
             pnlsMaxIter = 100000,
             msMaxIter = 100000,
             tolerance = 0.00001,
             msTol = 0.00001,
             pnlsTol = 0.00001,
             niterEM = 10000,
             opt = "optim",optimMethod = 'BFGS',
             minScale = 1e-20,
             gradient = T,
             msVerbose = T
)

###########################################################################################################
# Minimum and maximum values for PMIe
a1 <- min(hypostasis$PMIe) 
a2 <- max(hypostasis$PMIe) 

##########################################################################################################
# Adaptation of the Muggeo et al. (2014) algorithm

# Step 1: Initialization

# A: Setting an initial value for the changepoint \widetilde{\psi}_i = psii = psi
# for all i = 1,2,...,n
psi <- 9


# B: Initializing the variables \widetilde{U} = U = (t-psi) I (t_{ij} <= psi_i) 
hypostasis$U <- ifelse(hypostasis$PMIe <= psi,(hypostasis$PMIe ), psi)

# Fitting model zero

fit <- lme(difARPL ~  U , random = ~  U|subject, data = hypostasis)
summary(fit)
rand.coefpred <- (fit$coefficients)

# Step 2:

# Setting psi = psii and computing deltai, aprox.lambda0im, aprox G0i, Lambda and psi
lambda <- log((psi - a1)/(a2 - psi))
psi <- (a1 + a2*exp(lambda))/(1 + exp(lambda)) 

# beta
beta <- as.numeric(rand.coefpred$fixed[1])
betai <- beta + ranef(fit)[,1]
Betai <- rep( betai,times = as.numeric(table(fit$groups)) ) 

# Matrices hij, g, offset
hij <- (1 - I(as.numeric(hypostasis$PMIe <= psi)))
g <- (exp(lambda)*(a2 - a1))/((1 + exp(lambda))^2)
hypostasis$G0 <- as.numeric(Betai*hij*g )
Offset <- -lambda*hypostasis$G0 
hypostasis$difARPLnew <- hypostasis$difARPL - Offset

# Matrix U 
hypostasis$U <- ifelse(hypostasis$PMIe <= psi, hypostasis$PMIe  , psi) 
hypostasis <- groupedData(difARPL ~  U  |Subject, data = hypostasis)

# Fitting the model yij = betai(psi(lambda) + (xij - psi(lambda))I(xij <= psi(lambda))) + eij
fit <- lme(difARPLnew ~  U + G0 - 1 , random = ~  U + G0 - 1 |subject,
           control = Ctrl,method = 'ML' ,data = hypostasis)
summary(fit)
rand.coefpred <- (fit$coefficients)

# Step 3:

# Setting psi = psii  and computing  deltai, aprox.kappa0im, approx G0i, Lambda and psi
lambda <- as.numeric(rand.coefpred$fixed[2])
lambdai <- lambda + ranef(fit)[,2]
Lambdai <- rep(lambdai, times = as.numeric(table(fit$groups)) )
psi <- (a1 + a2*exp( lambda ))/(1 + exp(lambda))
psii <- (a1 + a2*exp( lambdai ))/(1 + exp(lambdai))

#Psii <- (a1 + a2*exp( Lambdai ))/(1 + exp(Lambdai))
Psii <- rep(psii, times = as.numeric(table(fit$groups)) )

# beta
beta <- as.numeric(rand.coefpred$fixed[1])
betai <- beta + ranef(fit)[,1]
Betai <- rep( betai,times = as.numeric(table(fit$groups)) ) 

# Matrices hij, g, offset
hij <- (1 - I(as.numeric(hypostasis$PMIe <= Psii)))
g <- (exp(Lambdai)*(a2 - a1))/((1 + exp(Lambdai))^2)
hypostasis$G0 <- as.numeric(Betai*hij*g )
Offset <- -Lambdai*hypostasis$G0 
hypostasis$difARPLnew <- hypostasis$difARPL - Offset

# Matrix U 
hypostasis$U <- ifelse(hypostasis$PMIe <= Psii, hypostasis$PMIe , Psii)

epsilon <- 1
LogLik0 <- abs(fit$logLik)
iter <- 0
norm.psii0 <- norm(as.matrix(psii))

# Step 3:
while (abs(epsilon) > 1e-3) {
  
  fit <- lme(difARPLnew ~ U  + G0 - 1, random = ~ U + G0 - 1|subject,
             control = Ctrl,method = 'REML',data = hypostasis)
  summary(fit)
  rand.coefpred <- (fit$coefficients)

  # New values for Lambda and psi
  lambda <- as.numeric(rand.coefpred$fixed[2])
  lambdai <- lambda + ranef(fit)[,2]
  Lambdai <- rep(lambdai, times = as.numeric(table(fit$groups)) )
  psi <- (a1 + a2*exp( lambda ))/(1 + exp(lambda))
  psii <- (a1 + a2*exp( lambdai ))/(1 + exp(lambdai))
  Psii <- rep(psii, times = as.numeric(table(fit$groups)) )
  
  # beta
  beta <- as.numeric(rand.coefpred$fixed[1])
  betai <- beta + ranef(fit)[,1]
  Betai <- rep( betai,times = as.numeric(table(fit$groups)) ) 
  
  # Matrix hij, g, offset
  hij <- (1 - I(as.numeric(hypostasis$PMIe <= Psii)))
  g <- (exp(Lambdai)*(a2 - a1))/((1 + exp(Lambdai))^2)
  hypostasis$G0 <- as.numeric(Betai*hij*g )
  Offset <- -Lambdai*hypostasis$G0 
  hypostasis$difARPLnew <- hypostasis$difARPL - Offset
  
  # Matrix U 
    hypostasis$U <- ifelse(hypostasis$PMIe <= Psii, hypostasis$PMIe , Psii)
  
  LogLik <- abs(fit$logLik)
  epsilon <-  abs(LogLik - LogLik0) / LogLik0
  norm.psii0 <- norm(as.matrix(psii))
  LogLik0 <- LogLik
    cat( "\n")
    cat("iteration = ", iter <- iter + 1,'epsilon= ',epsilon,'psi= ',psi, "\n")
}

# Computation of  beta, lambda, psi, asymp and Var(asymp) 

psi
asymp <- beta*psi
asympi <- betai*psii

# Asymptote gradient  asymp(widehat(beta),psi(widehat(lambda)))
grad.asymp <- as.matrix(c(psi,beta*(exp(lambda)*(a2 - a1))/((1 + exp(lambda))^2)),1,2)

# Asymptotic  variance
var.asymp <- t(grad.asymp) %*% fit$varFix %*% grad.asymp
std.error <- sqrt(var.asymp)

# Individual predicted plots

m0 <- fit
a1new <- min(hypostasis$difARPLnew)
a2new <- max(hypostasis$difARPLnew)
pred <- with(hypostasis,
             expand.grid(xij = seq(1, a2new,by = 0.5),
                         subject = (unique(subject))))
cutpoint <- rep(psii,each = 140)
pred$U <- ifelse(pred$xij <= cutpoint,pred$xij ,cutpoint)

pred$G0 <- rep(0,dim(pred)[1]) 

# Commands to include zero slope line after cutpoint
pred$difARPL <- ifelse(pred$U <= cutpoint,
                       predict(m0,newdata = pred,level = 1), 0)
pred$subjectf <- as.factor(pred$subject)

#Plot
xyplot(PMIe ~ difARPL | factor(subject), data = hypostasis,strip = T,
       key = list(columns = 1, text = list(c( "SS")),lty = 2,
                  lines = list(type = "l", col = c("black"),lty = 2)),
       ylim = c(0,25),xlim = c(0,99),  xlab="difARPL", ylab="Predicted PMIe",
       par.settings = standard.theme(quartz,color = F),as.table = TRUE) +
  as.layer(xyplot(pred$xij~ difARPL|subjectf, data = pred, type = "l",lty = 2, 
                  col = 1, order = T))

######################################################################################
