##############################################################################################################
# R code for fitting von BERTALANFFY models to the hypostasis data                                           #
# Manuscript: Fitting non-linear mixed models to messy longitudinal data: a case studey involving estimation #
#             of post mortem intervlas                                                                       #
# Authors: Francisco M.M. Rocha, Carmen D.S. Andre, Julio M Singer and Talita Zerbini                        #
# Updated June 14, 2017                                                                                      #
##############################################################################################################


library(MASS)
library(lattice)
library(latticeExtra)
library(gdata)
library(nlme)
library(car)
library(gdata)
library(Matrix)

#==============================================================================#
#
# Set the working directory
# setwd("~/Documents/Artigos/BJPS/ComandosR")
#==============================================================================#

# Read data from Excel spreadsheet
hypostasis <- read.xls("/home/jmsinger/Desktop/hypostasis.xls", sheet='data', method="tab")
#
# hypostasis <- read.xls("hypostasis.xls", sheet = 'data', method = "tab")


hypostasis$subject <- as.integer(hypostasis$Subject)

# Individual profile plots
xyplot(difARPL~PMIe | Subject, data = hypostasis, type = "b",
       par.settings = standard.theme(quartz, color = F), strip = TRUE)
xyplot(difARPL~PMIe, groups = Subject, data = hypostasis,type = "b", strip = TRUE)

###################################################################################################################

hypostasis <- groupedData(difARPL ~ PMIe|Subject,data = hypostasis)

##################################################################################################################
# von Bertalanffy model with random asymptote (asym) 
# Model as in the manuscript: y=alpha(1-exp(-beta*PMIe))
# Modified model to guarantee positive estimate for beta: y=alpha(1-exp(-exp(phi)*PMIe)); here beta=exp(phi)
##################################################################################################################

Ctrl.nlin1 <- list(pnlsMaxIter = 150,opt = "nlminb",gradient = T, msVerbose = T,
                   maxIter = 1000,niterEM = 500,msMaxIter = 1000,pnlsTol = 0.0001,
                   returnObject = TRUE,minScale = 0.00001,
                   tolerance = 1e-8)
fit1r.nlin <- nlme(difARPL ~ SSasympOrig(PMIe,asym,beta), data = hypostasis,
                start = c(asym = 45, beta = -1.5),fixed = asym + beta ~ 1,
                random = asym  ~ 1 | subject,control = Ctrl.nlin1, method = 'REML')

summary(fit1r.nlin)
result1 <- summary(fit1r.nlin); names(result1)
summary(fit1r.nlin)$tTable

# Individual predicted plots
plot(augPred(fit1r.nlin, level = 1),
     par.settings = standard.theme(quartz, color = F),
     xlab = "PMI", ylab = "Predicted difARPL", ylim = c(0, 100), lty=2,
     key = list(columns = 1, text = list(c("SS")),
     lines = list(col = 1, type = "l", lty = 2)))

################################################################################################################
# von Bertalanffy model with random intrinsic growth parameter (beta)
################################################################################################################

Ctrl.nlin1 <- list(pnlsMaxIter = 50,opt = "nlminb",gradient = T, msVerbose = T,
                   maxIter = 1000,niterEM = 500,msMaxIter = 1000,pnlsTol = 0.0001,
                   returnObject = TRUE,minScale = 0.00001,
                   tolerance = 1e-8)



fit2r.nlin <- nlme(difARPL ~ SSasympOrig(PMIe,asym,beta), data = hypostasis, 
                 start = c(asym = 41.1, beta = -0.7),
                 fixed = asym + beta ~ 1, random = beta  ~ 1|Subject, 
                 control = Ctrl.nlin1, method = 'REML')
summary(fit2r.nlin)
 
result2 <- summary(fit2r.nlin); names(result2)
summary(fit2r.nlin)$tTable

# Individual predicted plots
plot(augPred(fit2r.nlin, level = 1), 
     par.settings = standard.theme(quartz, color = F), lty=2,
     xlab = "PMI",ylab = "Predicted DifARPL",
     key = list(columns = 2, text = list(c("SS")),
     lines = list(type = "l", col = c("black"), lty =2)))

################################################################################################################
# von Bertalanffy model with random asymptote (asym) and random intrinsic growth parameter (beta)
################################################################################################################

Ctrl.nlin3 <- list(pnlsMaxIter = 50,opt = "nlminb",gradient = T, msVerbose = T,
     maxIter = 1000,niterEM = 5000,msMaxIter = 1000,pnlsTol = 0.1,
     returnObject = TRUE,minScale = 0.000000001,
     tolerance = 1e-8)
fit3r.nlin <- nlme(difARPL ~ SSasympOrig(PMIe,asym,beta),data = hypostasis,
                 start = c(asym = 20, beta = -1.5), fixed = asym + beta ~ 1,
                 control = Ctrl.nlin3,
                 random = asym + beta ~ 1 | Subject, method = 'REML')
summary(fit3r.nlin)
result3 <- summary(fit3r.nlin);names(result3)
summary(fit3r.nlin)$tTable
VarCorr(fit3r.nlin)
ranef(fit3r.nlin)[,1] + summary(fit3r.nlin)$tT[1,1]

# Individual predicted plots
plot(augPred(fit3r.nlin, level = 1), 
     par.settings = standard.theme(quartz, color = F), lty = 2,
     xlab = "PMI", ylab = "Predicted DifARPL", as.table = TRUE,
     key = list(columns = 1, text = list(c("SS")),
     lines = list(type = "l", col = c("black"), lty = 2)))

#########################################################################################################
# Individual predicted plots (included in the paper)

m0 <- fit3r.nlin
pred <- with(hypostasis,
             expand.grid(PMIe = seq(1, 21, by = 0.5),
                         subject = (unique(subject))))

pred$difARPL <- predict(m0, newdata = pred, level = 1)
pred$Subject <- as.factor(pred$subject)


#Plot
xyplot(PMIe ~ difARPL | factor(subject), data = hypostasis, strip = T,
       key = list(columns = 1, text = list(c("SS")), lty = 2,
                  lines = list(type = "l", col = c("black"),lty = 2)),
       ylim = c(0,25),xlim = c(0,99),
      par.settings = standard.theme(quartz, color = F), as.table = TRUE) +
      as.layer(xyplot(pred$PMIe ~ pred$difARPL | Subject, data = pred, type = "l", lty = 2, 
                  col = 1, order = T))
# 

######################################################################################




################################################################################################################



